/* react.c             ReAction GUI                                           */

/*----------------------------------------------------------------------------*/
/* System Includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/intuition.h>

	#define __NOLIBBASE__
	#define ALL_REACTION_CLASSES
	#include <reaction/reaction.h>

	#include <clib/alib_protos.h>
	#include <clib/debug_protos.h>
	#include <string.h>
	#include <stdarg.h>

/*----------------------------------------------------------------------------*/
/* Constants & Macros                                                         */
/*----------------------------------------------------------------------------*/

	#define GID_CHOOSER1    11
	#define GID_CHOOSER2    22
	#define GID_LIST        33

/*----------------------------------------------------------------------------*/
/* Type definitions                                                           */
/*----------------------------------------------------------------------------*/

	struct gui {
		struct MsgPort *appport;
		Object *winobj;
		struct Gadget *layout;
		struct Gadget *chooser1;
		struct Gadget *chooser2;
		struct Screen *scr;
		struct Window *win;
		struct List chooser1_labels;
		struct List chooser2_labels;
		};

/*----------------------------------------------------------------------------*/
/* Global variables                                                           */
/*----------------------------------------------------------------------------*/

	const char Title[] = "Big Chooser Example";

	struct ClassLibrary *WindowBase;      Class *WindowClass   = NULL;
	struct ClassLibrary *LabelBase;       Class *LabelClass    = NULL;
	struct ClassLibrary *LayoutBase;      Class *LayoutClass   = NULL;
	struct ClassLibrary *SpaceBase;       Class *SpaceClass    = NULL;
	struct ClassLibrary *ButtonBase;      Class *ButtonClass   = NULL;
	struct ClassLibrary *ChooserBase;     Class *ChooserClass  = NULL;
	struct ClassLibrary *ListBrowserBase; Class *ListBrowserClass  = NULL;

#ifdef __amigaos4__
	#define putchproc NULL
#else
	static const ULONG putchproc[] = {0x16c04e75};	/* move.b d0,(a3)+ ; rts */
#endif

/*----------------------------------------------------------------------------*/
/* Open a class library                                                       */
/*----------------------------------------------------------------------------*/

	struct ClassLibrary *myOpenClass (STRPTR name, ULONG version, Class **cl_ptr, char *errortext)

	{
	struct ClassLibrary *LayoutBase;

	if ((LayoutBase = (struct ClassLibrary *) OpenLibrary (name, version)))
		{
	#ifdef __amigaos4__
		*cl_ptr = LayoutBase->cl_Class;
	#else
		*cl_ptr = LAYOUT_GetClass();
	#endif
		}
	else
		{
		APTR args[] = {name,(APTR)version};
		RawDoFmt (version ? "%s V%lu\n" : "%s\n",args,(APTR)putchproc,&errortext[strlen(errortext)]);
		}

	return (LayoutBase);
	}

/*----------------------------------------------------------------------------*/
/* Close a class library                                                      */
/*----------------------------------------------------------------------------*/

	VOID myCloseClass (struct ClassLibrary *cl)

	{
	if (cl)
		CloseLibrary (&cl->cl_Lib);
	}

/*----------------------------------------------------------------------------*/
/* Display text or IoErr in an easy requester                                 */
/*----------------------------------------------------------------------------*/

	void fault_requester (const char *text,...)

	{
	struct EasyStruct es;
	ULONG idcmp = 0;

	es.es_StructSize   = sizeof(struct EasyStruct);
	es.es_Flags        = 0;
	es.es_Title        = (STRPTR)Title;
	es.es_TextFormat   = (STRPTR)text;
	es.es_GadgetFormat = "Cancel";

	EasyRequestArgs (NULL,&es,&idcmp,(&text)+1);
	}

/*----------------------------------------------------------------------------*/
/* Open all required class libraries                                          */
/*----------------------------------------------------------------------------*/

	BOOL open_libs (void)

	{
	char textbuffer[1024];
	char *errortext;

	strcpy (textbuffer,"The following required classes are missing:\n\n");
	errortext = &textbuffer[strlen(textbuffer)];

	WindowBase      = myOpenClass ("window.class",               0, &WindowClass,      errortext);
	LabelBase       = myOpenClass ("images/label.image",         0, &LabelClass,       errortext);
	LayoutBase      = myOpenClass ("gadgets/layout.gadget",      0, &LayoutClass,      errortext);
	SpaceBase       = myOpenClass ("gadgets/space.gadget",       0, &SpaceClass,       errortext);
	ButtonBase      = myOpenClass ("gadgets/button.gadget",      0, &ButtonClass,      errortext);
	ChooserBase     = myOpenClass ("gadgets/chooser.gadget",     0, &ChooserClass,     errortext);
	ListBrowserBase = myOpenClass ("gadgets/listbrowser.gadget", 0, &ListBrowserClass, errortext);

	if (*errortext)
		{
		fault_requester (textbuffer);
		return (FALSE);
		}

	return (TRUE);
	}

/*----------------------------------------------------------------------------*/
/* Close all opened class libraries                                           */
/*----------------------------------------------------------------------------*/

	void close_libs (void)

	{
	myCloseClass (ListBrowserBase);
	myCloseClass (ChooserBase);
	myCloseClass (ButtonBase);
	myCloseClass (SpaceBase);
	myCloseClass (LayoutBase);
	myCloseClass (LabelBase);
	myCloseClass (WindowBase);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	ULONG select_from_list (struct List *list,struct Window *relwin,struct Gadget *relgad)

	{
	ULONG id = 0xFFFFFFFF;
	Object *winobj;
	struct Gadget *listgad;

	if ((winobj = NewObject (WindowClass, NULL,
			WA_PubScreen, relwin->WScreen,
			WA_Flags, WFLG_BORDERLESS | WFLG_ACTIVATE,
			WA_Left, relwin->LeftEdge + relgad->LeftEdge,
			WA_Top, relwin->TopEdge + relgad->TopEdge + relgad->Height,
			WA_Width, relgad->Width,
			WA_IDCMP, IDCMP_VANILLAKEY | IDCMP_INACTIVEWINDOW,

			WINDOW_Layout, (APTR) NewObject (LayoutClass, NULL,
				LAYOUT_Orientation, LAYOUT_ORIENT_VERT,
				LAYOUT_DeferLayout, TRUE,

				LAYOUT_AddChild, (listgad = (APTR) NewObject (ListBrowserClass, NULL,
					LISTBROWSER_Labels, list,
					LISTBROWSER_MinVisible, 15,
					GA_RelVerify, TRUE,
					GA_ID, GID_LIST,
					TAG_END)),

				TAG_END),
			TAG_END)))
		{
		struct Window *win;

		if ((win = (struct Window *) DoMethod (winobj,WM_OPEN)))
			{
			ULONG winsig;
			BOOL cont = TRUE;

			GetAttr (WINDOW_SigMask, winobj, &winsig);

			do	{
				ULONG sigs = Wait (winsig | SIGBREAKF_CTRL_C);

				if (sigs & SIGBREAKF_CTRL_C)
					cont = FALSE;

				if (sigs & winsig)
					{
					ULONG result;

					while ((result = DoMethod (winobj, WM_HANDLEINPUT, NULL)) != WMHI_LASTMSG)
						{
						switch (result & WMHI_CLASSMASK)
							{
						case WMHI_GADGETUP:
							switch (result & WMHI_GADGETMASK)
								{
							case GID_LIST:
								GetAttr (LISTBROWSER_Selected, (Object *)listgad, &id);
								cont = FALSE;
								break;
								}
							break;
						case WMHI_VANILLAKEY:
							switch (result & WMHI_KEYMASK)
								{
							case 0x1b:
								cont = FALSE;
								break;
								}
							break;
						case WMHI_INACTIVE:
							cont = FALSE;
							break;
							}
						}
					}
				}
			while (cont);
			}
		else
			Printf ("cannot open window\n");

		DisposeObject (winobj);
		}

	return (id);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void hide_gui (struct gui *gui,BOOL iconify)

	{
	if (gui->win)
		{
		if (iconify)
			DoMethod (gui->winobj,WM_ICONIFY);
		else
			DoMethod (gui->winobj,WM_CLOSE);

		gui->win = NULL;
		}

	if (gui->scr)
		{
		SetAttrs (gui->winobj,WA_PubScreen,NULL,TAG_END);
		UnlockPubScreen (NULL,gui->scr);
		gui->scr = NULL;
		}
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	BOOL show_gui (struct gui *gui)

	{
	if (gui->win)
		return (TRUE);

	if ((gui->scr = LockPubScreen (NULL)))
		{
		SetAttrs (gui->winobj,
			WA_PubScreen,gui->scr,
			TAG_END);

		if ((gui->win = (struct Window *) DoMethod (gui->winobj,WM_OPEN)))
			return (TRUE);
		}

	hide_gui (gui,FALSE);
	return (FALSE);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	BOOL make_chooser_labels (struct List *list,...)

	{
	va_list ap;
	char *text;
	struct Node *node;
	BOOL ok = TRUE;

	va_start(ap, list);
	while ((text = va_arg(ap, char *)))
		{
		if ((node = AllocChooserNode (CNA_Text, text, TAG_END)))
			AddTail (list,node);
		else
			ok = FALSE;
		}
	va_end(ap);

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void free_chooser_labels (struct List *list)

	{
	struct Node *node;

	while ((node = RemTail (list)))
		FreeChooserNode (node);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	BOOL make_listbrowser_labels (struct List *list,...)

	{
	va_list ap;
	char *text;
	struct Node *node;
	BOOL ok = TRUE;

	va_start(ap, list);
	while ((text = va_arg(ap, char *)))
		{
		if ((node = AllocListBrowserNode (1, LBNCA_Text, text, TAG_END)))
			AddTail (list,node);
		else
			ok = FALSE;
		}
	va_end(ap);

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void free_listbrowser_labels (struct List *list)

	{
	struct Node *node;

	while ((node = RemTail (list)))
		FreeListBrowserNode (node);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	void free_gui (struct gui *gui)

	{
	hide_gui (gui,FALSE);

	if (gui->winobj)
		{
		DisposeObject (gui->winobj);
		gui->winobj = NULL;
		}

	free_chooser_labels (&gui->chooser1_labels);
	free_listbrowser_labels (&gui->chooser2_labels);

	if (gui->appport)
		{
		DeleteMsgPort (gui->appport);
		gui->appport = NULL;
		}

	close_libs ();
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	struct Node *get_list_node (struct List *list,ULONG id)

	{
	struct Node *node, *next;

	if (id == 0xFFFFFFFF)
		return (NULL);

	for (node = list->lh_Head; (next = node->ln_Succ); node = next)
		{
		if (id == 0)
			return (node);
		id --;
		}

	return (NULL);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	BOOL init_gui (struct gui *gui)

	{
	memset (gui,0,sizeof(*gui));

	NewList (&gui->chooser1_labels);
	NewList (&gui->chooser2_labels);

	if (!open_libs())
		{
		free_gui (gui);
		return (FALSE);
		}

	if (!(gui->appport = CreateMsgPort()))
		{
		free_gui (gui);
		return (FALSE);
		}

	if (!make_chooser_labels (&gui->chooser1_labels,
			"One",
			"Two",
			"Three",
			"Four",
			"Five",
			NULL))
		{
		free_gui (gui);
		return (FALSE);
		}

	if (!make_listbrowser_labels (&gui->chooser2_labels,
			"Tokyo",
			"Delhi",
			"Shanghai",
			"So Paulo",
			"Mexico City",
			"Cairo",
			"Mumbai",
			"Beijing",
			"Dhaka",
			"Osaka",
			"New York City",
			"Karachi",
			"Buenos Aires",
			"Chongqing",
			"Istanbul",
			"Kolkata",
			"Manila",
			"Lagos",
			"Rio de Janeiro",
			"Tianjin",
			"Kinshasa",
			"Guangzhou",
			"Los Angeles",
			"Moscow",
			"Shenzhen",
			"Lahore",
			"Bangalore",
			"Paris",
			"Bogot",
			"Jakarta",
			"Chennai",
			"Lima",
			"Bangkok",
			"Seoul",
			"Nagoya",
			"Hyderabad",
			"London",
			"Tehran",
			"Chicago",
			"Chengdu",
			"Nanjing",
			"Wuhan",
			"Ho Chi Minh City",
			"Luanda",
			"Ahmedabad",
			"Kuala Lumpur",
			"Xi'an",
			"Hong Kong",
			"Dongguan",
			"Hangzhou",
			"Foshan",
			"Shenyang",
			"Riyadh",
			"Baghdad",
			"Santiago",
			"Surat",
			"Madrid",
			"Suzhou",
			"Pune",
			"Harbin",
			"Houston",
			"Dallas",
			"Toronto",
			"Dar es Salaam",
			"Miami",
			"Belo Horizonte",
			"Singapore",
			"Philadelphia",
			"Atlanta",
			"Fukuoka",
			"Khartoum",
			"Barcelona",
			"Johannesburg",
			"Saint Petersburg",
			"Qingdao",
			"Dalian",
			"Washington, D.C.",
			"Yangon",
			"Alexandria",
			"Jinan",
			"Guadalajara",
			NULL))
		{
		free_gui (gui);
		return (FALSE);
		}

	if (!(gui->winobj = NewObject (WindowClass, NULL,
			WA_Title, Title,
			WA_Flags, WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_SIZEGADGET | WFLG_SIZEBBOTTOM | WFLG_ACTIVATE,
			WA_IDCMP, IDCMP_VANILLAKEY,

			WINDOW_AppPort, gui->appport,
			WINDOW_IconifyGadget, TRUE,

			WA_Width,  400,
			WA_Height, 200,

			WINDOW_Position, WPOS_CENTERSCREEN,

			WINDOW_Layout, (gui->layout = (APTR) NewObject (LayoutClass, NULL,
				LAYOUT_Orientation, LAYOUT_ORIENT_VERT,
				LAYOUT_DeferLayout, TRUE,
				LAYOUT_SpaceOuter, TRUE,

				LAYOUT_AddChild, NewObject (SpaceClass, NULL,
					TAG_END),

				LAYOUT_AddChild, NewObject (LayoutClass, NULL,
					LAYOUT_Orientation, LAYOUT_ORIENT_HORIZ,

					LAYOUT_AddChild, (gui->chooser1 = (APTR) NewObject (ChooserClass, NULL,
						CHOOSER_Labels, &gui->chooser1_labels,
						CHOOSER_DropDown, TRUE,
						CHOOSER_Title, "Select Number",
						GA_RelVerify, TRUE,
						GA_ID, GID_CHOOSER1,
						TAG_END)),

					LAYOUT_AddChild, (gui->chooser2 = (APTR) NewObject (ButtonClass, NULL,
						GA_Text, "Select City",
						GA_RelVerify, TRUE,
						GA_ID, GID_CHOOSER2,
						TAG_END)),

					TAG_END),
				CHILD_WeightedHeight, 0,

				LAYOUT_AddChild, NewObject (SpaceClass, NULL,
					TAG_END),


				TAG_END)),
			TAG_END)))
		{
		free_gui (gui);
		return (FALSE);
		}

	return (TRUE);
	}

/*----------------------------------------------------------------------------*/
/*                                                                            */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	struct gui gui;

	if (init_gui (&gui))
		{
		if (show_gui (&gui))
			{
			ULONG winsig;
			BOOL cont = TRUE;

			GetAttr (WINDOW_SigMask, gui.winobj, &winsig);

			do	{
				ULONG sigs = Wait (winsig | SIGBREAKF_CTRL_C);

				if (sigs & SIGBREAKF_CTRL_C)
					cont = FALSE;

				if (sigs & winsig)
					{
					ULONG result;
					ULONG id;
					struct Node *node;
					char *label;

					while ((result = DoMethod (gui.winobj, WM_HANDLEINPUT, NULL)) != WMHI_LASTMSG)
						{
						switch (result & WMHI_CLASSMASK)
							{
						case WMHI_GADGETUP:
							switch (result & WMHI_GADGETMASK)
								{
							case GID_CHOOSER1:
								GetAttr (CHOOSER_Selected, (Object *)gui.chooser1,&id);
								if ((node = get_list_node (&gui.chooser1_labels,id)))
									{
									GetChooserNodeAttrs (node,CNA_Text,&label,TAG_END);
									Printf ("number selected: %s\n",label);
									}
								break;
							case GID_CHOOSER2:
								id = select_from_list (&gui.chooser2_labels,gui.win,gui.chooser2);
								if ((node = get_list_node (&gui.chooser2_labels,id)))
									{
									GetListBrowserNodeAttrs (node,LBNCA_Text,&label,TAG_END);
									Printf ("city selected: %s\n",label);
									}
								break;
								}
							break;
						case WMHI_VANILLAKEY:
							switch (result & WMHI_KEYMASK)
								{
							case 0x1b:
								cont = FALSE;
								break;
								}
							break;
						case WMHI_ICONIFY:
							hide_gui (&gui,TRUE);
							break;
						case WMHI_UNICONIFY:
							if (!show_gui (&gui))
								cont = FALSE;
							break;
						case WMHI_CLOSEWINDOW:
							cont = FALSE;
							break;
							}
						}
					}
				}
			while (cont);
			}
		else
			Printf ("cannot open window\n");

		free_gui (&gui);
		}
	else
		Printf ("cannot create window object\n");

	return (RETURN_OK);
	}

/*----------------------------------------------------------------------------*/
/* End of source code                                                         */
/*----------------------------------------------------------------------------*/
